<?php
/*
Plugin Name: WooCommerce SagePay Direct Payment Gateway
Plugin URI: http://www.devicesoftware.com/wordpress-plugins/woocommerce-extensions/sagepaydirect/
Description: Extends WooCommerce with SagePay Direct payment gateway.
Version: 0.1.1
Author: swicks
Author URI: http://www.devicesoftware.com/wordpress-plugins/
*/

/*  Copyright 2012  Devicesoftware  (email : info@devicesoftwware.com) 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//Sage pay currently supported credit/debit cards
define('SAGEPAY_CARDTYPES', 'MC,MasterCard,VISA,VISA Credit,DELTA,VISA Debit,UKE,VISA Electron,MAESTRO,Maestro (Switch),AMEX,American Express,DC,Diner\'s Club,JCB,JCB Card,LASER,Laser');
    
//list of currencies this can easily be modified by adding/removing items    
define('SAGEPAY_CURRENCY', 'USD,US Dollar (USD),EURO,Euro,GBP,GB Pound (GBP)');

define('VPS_PROTOCOL', 2.23);

add_action('plugins_loaded', 'sagepaydirect_init', 0);
 
function sagepaydirect_init() 
{
    class WC_Sagepaydirect extends WC_Payment_Gateway{

        function __construct()
        {
            
            global $ds_woocommerce;
            $this->sagepay_params   = array();
            $this->id               = 'sagepaydirect';
            $this->method_title     = __('SagePay Direct', 'woocommerce');
            $this->icon             = WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/sagepay.png';
            $this->has_fields       = true;
            
            
            // gateway urls            
            $this->simulator_url    = 'https://test.sagepay.com/Simulator/VSPDirectGateway.asp';
            $this->test_url         = 'https://test.sagepay.com/gateway/service/vspdirect-register.vsp';
            $this->live_url         = 'https://live.sagepay.com/gateway/service/vspdirect-register.vsp';
            
            // load form fields
            $this->init_form_fields();

            // initialise settings
            $this->init_settings();
            
            // group & tidy available card types
            $cards = array();
            foreach ( $this->settings as $k => $v ) {
                if(preg_match('/^cardtype-/', $k)){
                    if($v == 'yes')
                        $cards[] = preg_replace('/^cardtype-/', '', $k);
                   //unset($this->settings[$k]);
                }                
            }
            
            // variables            
            $this->title            = $this->settings['title'];
            $this->description      = $this->settings['description'];
            $this->vendor_name      = $this->settings['vendor-name'];
            $this->mode             = $this->settings['mode'];        
            $this->tx_type          = $this->settings['tx-type'];
            $this->currency         = $this->settings['currency'];
            $this->vps_protocol     = VPS_PROTOCOL;
            $this->active_cards     = $cards;
            $this->debug            = $this->settings['debug'];    
            $this->debugemail       = $this->settings['debugemail'];
                       
            // actions
            add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            add_action('woocommerce_thankyou_sagepaydirect', array(&$this, 'thankyou_page'));
                        
        } // end __construct
        
        /**
         * Admin Panel Options 
         **/       
        public function admin_options()
        {
            ?>
            <h3><?php _e('SagePay Direct', 'woocommerce'); ?></h3>
            <p><?php _e('SagePay Direct communicates directly with SagePay\'s gateway keeping the look and feel the same throughout the transaction.', 'woocommerce'); ?></p>
            <table class="form-table">
            <?php           
                // generate the settings form.
                $this->generate_settings_html();
            ?>
            </table><!--/.form-table-->
            <?php
        } // end admin_options()        

        /**
         * Initialise Gateway Settings Form Fields
         */
        public function init_form_fields()
        {
            
            // mode options
            $mode_options = array( 'simulator' => 'Simulator', 'test' => 'Test', 'live' => 'Live');            
            
            // transaction options
            $tx_options = array('PAYMENT' => __('Payment', 'woocommerce'), 'DEFFERRED' => __('Deferred', 'woocommerce'), 'AUTHENTICATE' => __('Authenticate', 'woocommerce'));
            
            // add available currencies
            $currency_options=array();
            $available_currencies = explode(',', SAGEPAY_CURRENCY);
            for ($i=0; $i < count($available_currencies); $i+=2){
                $currency_options[$available_currencies[$i]] = $available_currencies[$i+1];
            }
            
            //  array to generate admin form
            $this->form_fields = array(
                'enabled' => array(
                                'title' => __( 'Enable/Disable', 'woocommerce' ), 
                                'type' => 'checkbox', 
                                'label' => __( 'Enable SagePay Direct', 'woocommerce' ), 
                                'default' => 'yes'
                            ), 
                'title' => array(
                                'title' => __( 'Title', 'woocommerce' ), 
                                'type' => 'text', 
                                'description' => __( 'This is the title displayed to the user during checkout.', 'woocommerce' ), 
                                'default' => __( 'SagePay Direct', 'woocommerce' )
                            ),
                'description' => array(
                                'title' => __( 'Description', 'woocommerce' ), 
                                'type' => 'textarea', 
                                'description' => __( 'This is the description which the user sees during checkout.', 'woocommerce' ), 
                                'default' => __("Payment via SagePay Gateway, you can pay by credit or debit card", 'woocommerce')
                            ),
                'vendor-name' => array(
                                'title' => __( 'Vendor Name', 'woocommerce' ), 
                                'type' => 'text', 
                                'description' => __( 'Please enter your vendor name provided by SagePay.', 'woocommerce' ), 
                                'default' => ''
                            ),
                'mode'      => array(
                                'title' => __('Mode Type', 'woocommerce'),
                                'type' => 'select',
                                'options' => $mode_options,
                                'description' => __( 'Select Simulator, Test or Live modes.', 'woocommerce' )
                            ),
                'tx-type'      => array(
                                'title' => __('Transition Type', 'woocommerce'),
                                'type' => 'select',
                                'options' => $tx_options,
                                'description' => __( 'Select Payment, Deferred or Authenticated.', 'woocommerce' )
                            ),
                'currency'  =>  array(
                                'title' => __('Gateway Currency', 'woocommerce'),
                                'type' => 'select',
                                'options' => $currency_options,
                                'description' => __( 'Select the currency you are using for this payment gateway.', 'woocommerce' )
                            )
                );
            
            
            // add available card types to the form field array    
            $available_cardtypes = explode(',', SAGEPAY_CARDTYPES);
            for ($i=0; $i < count($available_cardtypes); $i+=2){
                $this->form_fields['cardtype-' . $available_cardtypes[$i]] = array(
                    'type' => 'checkbox',
                    'label' => __( $available_cardtypes[$i+1], 'woocommerce' ), 
                    'default' => 'yes'
                );
                if($i == 0){
                     $this->form_fields['cardtype-' . $available_cardtypes[$i]]['title'] = __( 'Supported Cards', 'woocommerce' );
                }
            }
            $this->form_fields['debug'] = array(
                                'title' => __( 'Debug', 'woocommerce' ), 
                                'type' => 'checkbox', 
                                'label' => __( 'Enable logging ', 'woocommerce' ), 
                                'default' => 'no'
                            );
            $this->form_fields['debugemail'] = array(
                                'title' => __( 'Debug Email Address', 'woocommerce' ), 
                                'type' => 'text', 
                                'description' => __( 'Email address to catch debug information.  Only available in <b>Simulator</b> &amp; <b>Test</b> modes.', 'woocommerce' ), 
                                'default' =>  get_option('admin_email')
                            );
        } // end init_form_fields()
        
        /**
         * Payment fields for sagepay direct.
         **/
        function payment_fields() 
        {           
            // add available cards
            $card_select = "";
            $available_cardtypes = explode(',', SAGEPAY_CARDTYPES);
            for ($i=0; $i < count($available_cardtypes); $i+=2){
                if($this->settings['cardtype-' . $available_cardtypes[$i]] == 'yes')
                    $card_select .= "<option value='" . $available_cardtypes[$i] . "' >" . $available_cardtypes[$i+1] . "</option>\n";
            }
            
            // create month options and select current month as default
            $month_select = "";
            for ($i=0; $i < 12; $i++){
                $month = sprintf('%02d', $i+1);
                if($month == date('m'))
                    $select = 'selected ';
                else
                    $select = '';
                $month_select .= "<option value='" . $month . "' " . $select . ">" . $month . "</option>\n";
            }    
            
            // create options for valid from and expires on years
            $year_now = date('y');
            $from_year_select = "";
            $until_year_select = "";
            for($y = $year_now; $y > $year_now - 5; $y--){
                $year = sprintf('%02d', $y);
                $from_year_select .= "<option value='" . $year . "' " . $select . ">" . $year . "</option>\n";
            }
            for($y = $year_now; $y < $year_now + 7; $y++){
                $year = sprintf('%02d', $y);
                $until_year_select .= "<option value='" . $year . "' " . $select . ">" . $year . "</option>\n";
            }
                
            ?>
           
            <table style="width: 75%;">
            <tbody>
            <tr>
            <td><label for="sagepay_direct_fullname"><?php _e('Fullname', 'woocommerce') ?> <span class="required">*</span></label></td>
            <td><input id="sagepay_direct_fullname" class="input-text" type="text" value="" placeholder="<?php _e('Fullname', 'woocommerce') ?>" name="sagepay_direct_fullname"></td>
            </tr>
            <tr>
            <td><label for="sagepay_direct_cardtype"><?php _e('Card Type', 'woocommerce') ?> <span class="required">*</span></label></td>
            <td>
                <select id="sagepay_direct_cardtype" name="sagepay_direct_cardtype">
                  <?php echo $card_select; ?>
                </select>
            </td>
            </tr>
            <tr>
            <td><label for="sagepay_direct_cardnumber"><?php _e('Card Number', 'woocommerce') ?> <span class="required">*</span></label></td>
            <td><input id="sagepay_direct_cardnumber"  name="sagepay_direct_cardnumber" class="input-text" type="text" value="" placeholder="<?php _e('Card Number', 'woocommerce') ?>"></td>
            </tr>
            <tr>
            <td><label for="sagepay_direct_validfrom_mm"><?php _e('Valid From', 'woocommerce') ?> </label></td>
            <td><select id="sagepay_direct_validfrom_mm" name="sagepay_direct_validfrom_mm">
                <?php echo $month_select; ?>
            </select>&nbsp;<select id="sagepay_direct_validfrom_yy" name="sagepay_direct_validfrom_yy">
                <?php echo $from_year_select; ?>
            </select>
            </td>
            </tr>
            <tr>
            <td><label for="sagepay_direct_validuntil_mm"><?php _e('Valid Until', 'woocommerce') ?> <span class="required">*</span></label></td>
            <td><select id="sagepay_direct_validuntil_mm" name="sagepay_direct_validuntil_mm">
                <?php echo $month_select; ?>
            </select>&nbsp;<select id="sagepay_direct_validuntil_yy" name="sagepay_direct_validuntil_yy">
                <?php echo $until_year_select; ?>
            </select>
            </td>
            </tr>
            
            <tr>            
            <td><label for="sagepay_direct_cv2"><?php _e('CV2', 'woocommerce') ?> <span class="required">*</span></label></td>
            <td><input id="sagepay_direct_cv2"  name="sagepay_direct_cv2" size="4" class="input-text" type="text" value="" placeholder="<?php _e('CV2', 'woocommerce') ?>">
            <span><?php _e('The last 3 digits on the signature side.', 'woocommerce') ?></span></td>
            </tr>
            </tbody>
            </table> 

            <?php
        }// payment_fields

        /**
        * Validate payment fields
        */
        function validate_fields() 
        { 
            global $woocommerce;

            if(empty($_POST['sagepay_direct_fullname']))
                $woocommerce->add_error(__('SagePay Direct - Fullname required.', 'woocommerce'));
            if(empty($_POST['sagepay_direct_cardnumber']))
                $woocommerce->add_error(__('SagePay Direct - Card number required.', 'woocommerce'));
            if(empty($_POST['sagepay_direct_cv2']))
                $woocommerce->add_error(__('SagePay Direct - CV2 required.', 'woocommerce'));
            if(!$woocommerce->error_count())
            {
                // card information
                $this->add_param('CardHolder', $_POST['sagepay_direct_fullname']);
                $this->add_param('CardNumber', $_POST['sagepay_direct_cardnumber']);
                $this->add_param('StartDate', $_POST["sagepay_direct_validfrom_mm"] . $_POST["sagepay_direct_validfrom_yy"]);
                $this->add_param('ExpiryDate', $_POST["sagepay_direct_validuntil_mm"] . $_POST["sagepay_direct_validuntil_yy"]);
                $this->add_param('CV2', $_POST['sagepay_direct_cv2']);
                $this->add_param('CardType', $_POST['sagepay_direct_cardtype']);
                                        
                $this->add_param('VPSProtocol', VPS_PROTOCOL);
                $this->add_param('Vendor', $this->vendor_name);
                $this->add_param('Description', $this->vendor_name);
                $this->add_param('Currency', $this->currency);
                $this->add_param('TxType', $this->tx_type);
                
                //generate a unique vendorTxCode
                $time_stamp = date("ymdHis");
                $rand_num = rand(0,32000)*rand(0,32000);
                $this->vendor_tx_code = $this->vendor_name . "-" . $time_stamp . "-" . $rand_num;
                $this->add_param('VendorTxCode', $this->vendor_tx_code);                
                
                $this->validated = TRUE;
            }
            else
                $this->validated = FALSE;
        } // end validate_fields
        
        /**
        * process payment
        * 
        * @param int $order_id
        */
        function process_payment( $order_id ) 
        {
            global $woocommerce;
            if($this->validated){
                // woocommerse order instance
                $order = new woocommerce_order( $order_id );
                
                // total amount
                $this->add_param('Amount', $order->order_total);
                
                // customer email
                $this->add_param('CustomerEMail', $order->billing_email);
                
                // billing details
                $this->add_param('BillingSurname', $order->billing_first_name);
                $this->add_param('BillingFirstnames', $order->billing_last_name);
                $this->add_param('BillingAddress1', $order->billing_address_1);
                $this->add_param('BillingAddress2', $order->billing_address_2);
                $this->add_param('BillingCity', $order->billing_city);
                if($order->billing_country == 'US')
                    $this->add_param('BillingState', $order->billing_state);
                else
                    $this->add_param('BillingState', '');
                $this->add_param('BillingPostCode', $order->billing_postcode);                    
                $this->add_param('BillingCountry', $order->billing_country);
                $this->add_param('BillingPhone', $order->billing_phone);
                
                // delivery details
                $this->add_param('DeliverySurname', $order->shipping_first_name);
                $this->add_param('DeliveryFirstnames', $order->shipping_last_name);
                $this->add_param('DeliveryAddress1', $order->shipping_address_1);
                $this->add_param('DeliveryAddress2', $order->shipping_address_2);
                $this->add_param('DeliveryCity', $order->shipping_city);
                if($order->shipping_country == 'US')
                    $this->add_param('DeliveryState', $order->shipping_state);
                else
                    $this->add_param('DeliveryState', '');
                $this->add_param('DeliveryPostCode', $order->shipping_postcode);                    
                $this->add_param('DeliveryCountry', $order->shipping_country);
                // woocommerce doesn't currently support a delivery phone number using billing phone number instead'
                $this->add_param('DeliveryPhone', $order->billing_phone);                
                                
                // process param string
                $this->param_string = "";
                foreach( $this->params as $key => $value ) {
                    $this->param_string .= "$key=" . urlencode( $value ) . "&";
                }
                $this->param_string = rtrim( $this->param_string, "& " );

                
                // Post back to get a response                
                
                // gateway url
                $gateway_url =  $this->mode . '_url';
                $this->gateway_url = $this->$gateway_url;
                
                // debug
                $this->send_debug_email( "Payment Gateway URL: " . $this->gateway_url . "\n\nPayment Gateway Request: " . print_r($this->params,true));

                
                // removed CURL: now using wp's remote post - disabled ssl verify and use the post method'
                
                $params = array( 
                    'body' => $this->param_string,
                    'method' => 'POST',
                    'sslverify' => FALSE
                );                
                $this->response_string = wp_remote_post($this->gateway_url, $params);

                /* 
                
                //set up URL & options
                $gateway_url =  $this->mode . '_url';
                $curl = curl_init($this->$gateway_url);
                curl_setopt($curl, CURLOPT_HEADER, 0);
                curl_setopt($curl, CURLOPT_POST, 1);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $this->param_string);
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($curl, CURLOPT_TIMEOUT,30);        
                //remove these last 2 lines if using an earlier version of CURL
                curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
                curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 1);        
                
                // execute curl
                $this->response_string = curl_exec($curl);
                */
                
                
                $this->response = array();   
                
                // split response into lines
                $lines = preg_split( '/\r\n|\r|\n/', $this->response_string['body'] );
                foreach($lines as $line){            
                    $key_value = preg_split( '/=/', $line );
                    if(count($key_value) == 2)
                        $this->response[trim($key_value[0])] = trim($key_value[1]);
                }
                               
                // additional information returned for debugging
                $this->response['VendorTxCode'] = $this->vendor_tx_code;
                
                // creation date & time
                $this->response['Created'] = date('Y-m-d H:i:s');
                
                
                // handle status
                switch($this->response['Status']){
                    case 'OK':
                    case 'REGISTERED':
                        $order->add_order_note( __('Sagepay Direct payment completed', 'woocommerce') . ' (Transaction ID: ' . $this->response['transaction_id'] . ')' );
                        $order->payment_complete();                    
                        
                        // empty woocommerce cart
                        $woocommerce->cart->empty_cart();
                        
                        $this->response['result'] = 'success';
                        $this->response['redirect'] = add_query_arg('key', $order->order_key, add_query_arg('order', $order_id, get_permalink(get_option('woocommerce_thanks_page_id'))));
                        break;

                    case 'MALFORMED':
                    case 'INVALID':
                    case 'ERROR':
                    case 'NOTAUTHED':
                    case 'REJECTED':
                    case '3DAUTH':
                    default:
                        if(isset($this->response['StatusDetail']))
                            $woocommerce->add_error($this->response['StatusDetail']);
                        else
                            $woocommerce->add_error($this->response['Status'] . __(' - unknown error.', 'woocommerce'));
                        $this->response['result'] = strtolower($this->response['Status']);                        
                        break;
                }
                
                
                // debug
                $this->send_debug_email( "Payment Gateway URL: " . $this->gateway_url . "\n\nResponse Status: " . $this->response['Status'] . "\n\nPayment Gateway Response:" . print_r($this->response,true));

                return $this->response;
                        
            }                    
        } // end process_payment
        
        /**
        * Thank you page
        * 
        */
        function thankyou_page() 
        {
            if ($this->description) echo wpautop(wptexturize($this->description));
        } // end thankyou_page
           
        /**
         * Send debug email
         * 
         * @param string $msg
         **/
        function send_debug_email( $msg)
        {
            if($this->debug=='yes' AND $this->mode!=live AND !empty($this->debugemail)){
                // send debugemail
                wp_mail( $this->debugemail, __('SagePay Direct Debug', 'woocommerce'), $msg );
            }
            
            
            
        } // end send_debug_email
        
        /**
        * add sagepay parameters for later processing
        * 
        * @param string $param
        * @param mixed $value
        */
        public function add_param($param, $value) {
            $this->params[$param] = $value;   
        } // end add_param
    }    
} // end sagepaydirect_init

function add_sagepay_direct( $methods ) 
{
    $methods[] = 'WC_Sagepaydirect'; 
    return $methods;
} // end add_sagepay_direct


add_filter('woocommerce_payment_gateways', 'add_sagepay_direct' );
?>